/*
 *  queue.h
 *
 *  Implementation of a generic queue type.
 *
 *  Kekoa Proudfoot
 *  12/12/96
 */

#include <stdio.h>
#include <stdlib.h>

#include "queue.h"
#include "debug.h"

typedef struct elem_t {
    struct elem_t *next;
    void          *data;
} elem_t;

void
queue_init(qqueue_t *queue)
{
    queue->head = queue->tail = NULL;
}

int
queue_empty(qqueue_t *queue)
{
    return (queue->head == NULL);
}

void
queue_add(qqueue_t *queue, void *data)
{
    elem_t *new;

    if ((new = (elem_t *)malloc(sizeof(elem_t))) == NULL) {
	debug_fatal("queue_add: malloc error");
    }

    new->next = NULL;
    new->data = data;

    if (queue->head == NULL)
	queue->head = queue->tail = new;
    else
	((elem_t *)queue->tail)->next = new;

    queue->tail = new;
}

void *
queue_remove(qqueue_t *queue)
{
    void   *data;
    elem_t *elem;

    if (!queue->head)
	return NULL;

    elem        = queue->head;
    data        = elem->data;
    queue->head = elem->next;

    free(elem);

    return data;
}

void *
queue_get_head(qqueue_t *queue)
{
    elem_t *elem;

    if (!queue->head)
	return NULL;

    elem        = queue->head;

    return elem->data;
}
